//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// Quick & dirty test program for the "range finder" Sides<T,N>
// template class (defined and implemented in cbuf.hh; see there for
// more)
// -------------------------------------------------------------------
// $Id: main.cxx,v 1.1 2003/09/23 11:21:24 loreti Exp $
// -------------------------------------------------------------------

#include <iostream>
#include <string>
#include <cstdlib>
#include <ctime>
#include "sides.hh"

int main(
  int    argc,
  char * argv[]
) {
  if (argc != 4) {
    std::cout << "Usage: " << argv[0] << " Max Nvalues Nprintout\n";
  } else {
    double   maxVal        = std::atof( argv[1] );
    unsigned nVal          = std::atoi( argv[2] );
    unsigned nPrt          = std::atoi( argv[3] );
    const char separator[] = "---------------------------------";

    std::srand( std::time(0) );

    std::cout << separator << " [ N = 1 ] " << separator << std::endl;

    Sides<double> sd;

    for (unsigned iVal = 1;   iVal <= nVal;   ++iVal) {
      double x  = std::rand();
      x        /= RAND_MAX;
      x        *= maxVal;

      sd.enter(x);

      if ((iVal % nPrt) == 0) {
        std::cout << iVal << ":\t [" << sd.getMin() << ", "
                  << sd.getMax() << "]\n";
      }
    }

    std::cout << separator << " [ N = 3 ] " << separator << std::endl;

    Sides<double,3> s2;

    for (unsigned iVal = 1;   iVal <= nVal;   ++iVal) {
      double x  = std::rand();
      x        /= RAND_MAX;
      x        *= maxVal;

      s2.enter(x);

      if ((iVal % nPrt) == 0) {
        std::cout << iVal << ":\t [";
        const std::multiset<double> * p = s2.getLo();
        std::multiset<double>::const_iterator it, ite;
        std::string s;
        for (it = p->begin(),  ite = p->end(),  s = "";
             it != ite;   ++it,  s = ", ") std::cout << s << *it;
        std::cout << " <-> ";
        p = s2.getHi();
        for (it = p->begin(),  ite = p->end(),  s = "";
             it != ite;   ++it,  s = ", ") std::cout << s << *it;
        std::cout << "]\n";
      }
    }
  }
  return 0;
}
